using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.CompilerServices;
using Esri.ArcGISRuntime.Data;
using Esri.ArcGISRuntime.Geometry;
using Esri.ArcGISRuntime.Location;
using Esri.ArcGISRuntime.Mapping;
using Esri.ArcGISRuntime.Security;
using Esri.ArcGISRuntime.Symbology;
using Esri.ArcGISRuntime.Tasks;
using Esri.ArcGISRuntime.UI;
using Windows.Storage;
using Esri.ArcGISRuntime;
using Esri.ArcGISRuntime.UI.Controls;

namespace TimeIssue
{
    /// <summary>
    /// Provides map data to an application
    /// </summary>
    public class MapViewModel : INotifyPropertyChanged
    {
        private MapView _mapView;

        public MapViewModel(MapView mapView)
        {
            this._mapView = mapView;
            this.InitMap();
        }

        private async Task InitMap()
        {




            this._mapView.Map = this._map;

            ArcGISTiledLayer basemapLyr = await this.GetBasemapLayer();
            this._map.Basemap.BaseLayers.Add(basemapLyr);

            FeatureLayer lyr = await this.GetFeatureLayer();
            this._map.OperationalLayers.Add(lyr);

            //not helping at all
            //this._map.MinScale = 200000;
            //this._map.MaxScale = 0;
        }


        private async Task<FeatureLayer> GetFeatureLayer()
        {
            StorageFile geodatabaseFile = await StorageFile.GetFileFromApplicationUriAsync(new Uri("ms-appx:///offlineFeatures.geodatabase"));            
            String geodatabaseFilePath = geodatabaseFile.Path;

            var gdb = await Geodatabase.OpenAsync(geodatabaseFilePath);
            //supposed to be one and only layer in gdb
            GeodatabaseFeatureTable gdbFeatureTable = gdb.GeodatabaseFeatureTables.First();
            FeatureLayer lyr = new FeatureLayer(gdbFeatureTable);
            if (lyr.SupportsTimeFiltering == true)
            {
                lyr.IsTimeFilteringEnabled = true;
            }

            if(lyr.LoadStatus != LoadStatus.Loaded)
            {
                await lyr.LoadAsync();
            }

            return lyr;
        }

        private async Task<ArcGISTiledLayer> GetBasemapLayer()
        {
            StorageFile tpkFile = await StorageFile.GetFileFromApplicationUriAsync(new Uri("ms-appx:///basemapTiles.tpk"));
            TileCache tCache = new TileCache(tpkFile.Path);
            await tCache.LoadAsync();

            ArcGISTiledLayer tiledLyr = new ArcGISTiledLayer(tCache);
          
            return tiledLyr;

        }





        private Map _map = new Map();

        /// <summary>
        /// Gets or sets the map
        /// </summary>
        public Map Map
        {
            get { return _map; }
            set { _map = value; OnPropertyChanged(); }
        }

        /// <summary>
        /// Raises the <see cref="MapViewModel.PropertyChanged" /> event
        /// </summary>
        /// <param name="propertyName">The name of the property that has changed</param>
        protected void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            var propertyChangedHandler = PropertyChanged;
            if (propertyChangedHandler != null)
                propertyChangedHandler(this, new PropertyChangedEventArgs(propertyName));
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
